@@ -15,27 +15,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-
+#define _FILE_OFFSET_BITS 64
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #ifdef WIN32
 #include <sys/timeb.h>
-#include <winsock.h>
 #include <process.h>
 #else
+#include <sys/types.h>
+#include <pwd.h>
 #include <sys/time.h>
-#include <fcntl.h>
-#include <dirent.h>
 #include <sys/utsname.h>
 #endif
+#include <fcntl.h>
+#include <dirent.h>
 #include <errno.h>
 #include "xchat.h"
+#include "xchatc.h"
 #include <ctype.h>
 #include "util.h"
 #include "../../config.h"
+
+#include "inet.h"
+
 #ifdef USING_FREEBSD
 #include <sys/sysctl.h>
 #endif
@@ -187,6 +193,12 @@ xchat_free (void *buf, char *file, int line)
 {
 	struct mem_block *cur, *last;
 
+	if (buf == NULL)
+	{
+		printf ("%s:%d \033[33mTried to free NULL\033[m\n", file, line);
+		return;
+	}
+
 	last = NULL;
 	cur = mroot;
 	while (cur)
@@ -248,19 +260,22 @@ file_part (char *file)
 void
 path_part (char *file, char *path, int pathlen)
 {
+	unsigned char t;
 	char *filepart = file_part (file);
+	t = *filepart;
 	*filepart = 0;
 	safe_strcpy (path, file, pathlen);
+	*filepart = t;
 }
 
 char *				/* like strstr(), but nocase */
 nocasestrstr (char *s, char *wanted)
 {
-	register const size_t len = strlen (wanted);
+	register const int len = strlen (wanted);
 
 	if (len == 0)
 		return (char *)s;
-	while (tolower(*s) != tolower(*wanted) || strncasecmp (s, wanted, len))
+	while (rfc_tolower(*s) != rfc_tolower(*wanted) || strncasecmp (s, wanted, len))
 		if (*s++ == '\0')
 			return (char *)NULL;
 	return (char *)s;
@@ -269,7 +284,6 @@ nocasestrstr (char *s, char *wanted)
 char *
 errorstring (int err)
 {
-	static char tbuf[16];
 	switch (err)
 	{
 	case -1:
@@ -277,16 +291,7 @@ errorstring (int err)
 	case 0:
 		return _("Remote host closed socket");
 #ifndef WIN32
-	case ECONNREFUSED:
-		return _("Connection refused");
-	case ENETUNREACH:
-	case EHOSTUNREACH:
-		return _("No route to host");
-	case ETIMEDOUT:
-		return _("Connection timed out");
-	case EADDRNOTAVAIL:
-		return _("Cannot assign that address");
-	case ECONNRESET:
+	}
 #else
 	case WSAECONNREFUSED:
 		return _("Connection refused");
@@ -298,11 +303,54 @@ errorstring (int err)
 	case WSAEADDRNOTAVAIL:
 		return _("Cannot assign that address");
 	case WSAECONNRESET:
-#endif
 		return _("Connection reset by peer");
 	}
-	sprintf (tbuf, "%d", err);
-	return tbuf;
+
+	/* can't use strerror() on Winsock errors! */
+	if (err >= WSABASEERR)
+	{
+		static char tbuf[384];
+		OSVERSIONINFO osvi;
+
+		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
+		GetVersionEx (&osvi);
+
+		/* FormatMessage works on WSA*** errors starting from Win2000 */
+		if (osvi.dwMajorVersion >= 5)
+		{
+			if (FormatMessageA (FORMAT_MESSAGE_FROM_SYSTEM |
+									  FORMAT_MESSAGE_IGNORE_INSERTS |
+									  FORMAT_MESSAGE_MAX_WIDTH_MASK,
+									  NULL, err,
+									  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+									  tbuf, sizeof (tbuf), NULL))
+			{
+				int len;
+				char *utf;
+
+				tbuf[sizeof (tbuf) - 1] = 0;
+				len = strlen (tbuf);
+				if (len >= 2)
+					tbuf[len - 2] = 0;	/* remove the cr-lf */
+
+				/* now convert to utf8 */
+				utf = g_locale_to_utf8 (tbuf, -1, 0, 0, 0);
+				if (utf)
+				{
+					safe_strcpy (tbuf, utf, sizeof (tbuf));
+					g_free (utf);
+					return tbuf;
+				}
+			}
+		}	/* ! if (osvi.dwMajorVersion >= 5) */
+
+		/* fallback to error number */
+		sprintf (tbuf, "%s %d", _("Error"), err);
+		return tbuf;
+	} /* ! if (err >= WSABASEERR) */
+#endif	/* ! WIN32 */
+
+	return strerror (err);
 }
 
 int
@@ -329,26 +377,46 @@ char *
 expand_homedir (char *file)
 {
 #ifndef WIN32
-	char *ret;
+	char *ret, *user;
+	struct passwd *pw;
 
 	if (*file == '~')
 	{
-		ret = malloc (strlen (file) + strlen (g_get_home_dir ()) + 1);
-		sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
+		if (file[1] != '\0' && file[1] != '/')
+		{
+			user = strdup(file);
+			if (strchr(user,'/') != NULL)
+				*(strchr(user,'/')) = '\0';
+			if ((pw = getpwnam(user + 1)) == NULL)
+			{
+				free(user);
+				return strdup(file);
+			}
+			free(user);
+			user = strchr(file, '/') != NULL ? strchr(file,'/') : file;
+			ret = malloc(strlen(user) + strlen(pw->pw_dir) + 1);
+			strcpy(ret, pw->pw_dir);
+			strcat(ret, user);
+		}
+		else
+		{
+			ret = malloc (strlen (file) + strlen (g_get_home_dir ()) + 1);
+			sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
+		}
 		return ret;
 	}
 #endif
 	return strdup (file);
 }
 
-unsigned char *
-strip_color (unsigned char *text)
+char *
+strip_color (char *text)
 {
 	int nc = 0;
 	int i = 0;
 	int col = 0;
 	int len = strlen (text);
-	unsigned char *new_str = malloc (len + 2);
+	char *new_str = malloc (len + 2);
 
 	while (len > 0)
 	{
@@ -390,7 +458,7 @@ strip_color (unsigned char *text)
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 
 static void
-get_cpu_info (int *mhz, int *cpus)
+get_cpu_info (double *mhz, int *cpus)
 {
 
 #ifdef USING_LINUX
@@ -483,7 +551,7 @@ get_cpu_str (void)
 	static char verbuf[64];
 	OSVERSIONINFO osvi;
 	SYSTEM_INFO si;
-	int mhz;
+	double mhz;
 
 	osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
 	GetVersionEx (&osvi);
@@ -491,10 +559,14 @@ get_cpu_str (void)
 
 	mhz = get_mhz ();
 	if (mhz)
-		sprintf (verbuf, "Windoze %ld.%ld [i%1d86/%dMHz]",
-			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, mhz);
-	else
-		sprintf (verbuf, "Windoze %ld.%ld [i%1d86]",
+	{
+		double cpuspeed = ( mhz > 1000 ) ? mhz / 1000 : mhz;
+		const char *cpuspeedstr = ( mhz > 1000 ) ? "GHz" : "MHz";
+		sprintf (verbuf, "Windows %ld.%ld [i%d86/%.2f%s]",
+					osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, 
+					cpuspeed, cpuspeedstr);
+	} else
+		sprintf (verbuf, "Windows %ld.%ld [i%d86]",
 			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel);
 
 	return verbuf;
@@ -506,7 +578,8 @@ char *
 get_cpu_str (void)
 {
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
-	int mhz, cpus;
+	double mhz;
+	int cpus;
 #endif
 	struct utsname un;
 	static char *buf = NULL;
@@ -521,10 +594,14 @@ get_cpu_str (void)
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 	get_cpu_info (&mhz, &cpus);
 	if (mhz)
+	{
+		double cpuspeed = ( mhz > 1000 ) ? mhz / 1000 : mhz;
+		const char *cpuspeedstr = ( mhz > 1000 ) ? "GHz" : "MHz";
 		snprintf (buf, 128,
-					 (cpus == 1) ? "%s %s [%s/%dMHz]" : "%s %s [%s/%dMHz/SMP]",
-					 un.sysname, un.release, un.machine, mhz);
-	else
+					(cpus == 1) ? "%s %s [%s/%.2f%s]" : "%s %s [%s/%.2f%s/SMP]",
+					un.sysname, un.release, un.machine,
+					cpuspeed, cpuspeedstr);
+	} else
 #endif
 		snprintf (buf, 128, "%s %s [%s]", un.sysname, un.release, un.machine);
 
@@ -568,7 +645,7 @@ int match(const char *mask, const char *string)
 	if (*m == '?' || *m == '*')
 	  ch = *m++;
       default:
-	if (tolower(*s) != tolower(ch))
+	if (rfc_tolower(*s) != rfc_tolower(ch))
 	  return 0;
       case '?':
 	if (!*s++)
@@ -598,8 +675,8 @@ got_star:
 break_while:
   if (!ch)
     return 1;			/* mask ends with '*', we got it */
-  ch = tolower(ch);
-  while (tolower(*s++) != ch)
+  ch = rfc_tolower(ch);
+  while (rfc_tolower(*s++) != ch)
     if (!*s)
       return 0;
   bs = s;			/* Next try start from here */
@@ -615,7 +692,7 @@ break_while:
 	if (*m == '?' || *m == '*')
 	  ch = *m++;
       default:
-	if (tolower(*s) != tolower(ch))
+	if (rfc_tolower(*s) != rfc_tolower(ch))
 	{
 	  m = bm;
 	  s = bs;
@@ -635,13 +712,12 @@ break_while:
   return 1;
 }
 
-int
+void
 for_files (char *dirname, char *mask, void callback (char *file))
 {
 	DIR *dir;
 	struct dirent *ent;
 	char *buf;
-	int i = 0;
 
 	dir = opendir (dirname);
 	if (dir)
@@ -652,7 +728,6 @@ for_files (char *dirname, char *mask, void callback (char *file))
 			{
 				if (match (mask, ent->d_name))
 				{
-					i++;
 					buf = malloc (strlen (dirname) + strlen (ent->d_name) + 2);
 					sprintf (buf, "%s/%s", dirname, ent->d_name);
 					callback (buf);
@@ -662,30 +737,36 @@ for_files (char *dirname, char *mask, void callback (char *file))
 		}
 		closedir (dir);
 	}
-	return i;
 }
 
-void
+/*void
 tolowerStr (char *str)
 {
 	while (*str)
 	{
-		*str = tolower (*str);
+		*str = rfc_tolower (*str);
 		str++;
 	}
-}
+}*/
 
-/* thanks BitchX */
+/* This is originally from BitchX, but since added missing TLDs and
+   changed to a binary search */
+
+typedef struct
+{
+	char *code;
+	char *country;
+} Domain;
+
+static int
+country_compare (const void *a, const void *b)
+{
+	return strcasecmp (a, ((Domain *)b)->code);
+}
 
 char *
 country (char *hostname)
 {
-	typedef struct _domain
-	{
-		char *code;
-		char *country;
-	}
-	Domain;
 	static const Domain domain[] = {
 		{"AD", N_("Andorra") },
 		{"AE", N_("United Arab Emirates") },
@@ -698,8 +779,10 @@ country (char *hostname)
 		{"AO", N_("Angola") },
 		{"AQ", N_("Antarctica") },
 		{"AR", N_("Argentina") },
+		{"ARPA", N_("Reverse DNS") },
 		{"AS", N_("American Samoa") },
 		{"AT", N_("Austria") },
+		{"ATO", N_("Nato Fiel") },
 		{"AU", N_("Australia") },
 		{"AW", N_("Aruba") },
 		{"AZ", N_("Azerbaijan") },
@@ -711,6 +794,7 @@ country (char *hostname)
 		{"BG", N_("Bulgaria") },
 		{"BH", N_("Bahrain") },
 		{"BI", N_("Burundi") },
+		{"BIZ", N_("Businesses"), },
 		{"BJ", N_("Benin") },
 		{"BM", N_("Bermuda") },
 		{"BN", N_("Brunei Darussalam") },
@@ -733,6 +817,7 @@ country (char *hostname)
 		{"CM", N_("Cameroon") },
 		{"CN", N_("China") },
 		{"CO", N_("Colombia") },
+		{"COM", N_("Internic Commercial") },
 		{"CR", N_("Costa Rica") },
 		{"CS", N_("Former Czechoslovakia") },
 		{"CU", N_("Cuba") },
@@ -747,6 +832,7 @@ country (char *hostname)
 		{"DO", N_("Dominican Republic") },
 		{"DZ", N_("Algeria") },
 		{"EC", N_("Ecuador") },
+		{"EDU", N_("Educational Institution") },
 		{"EE", N_("Estonia") },
 		{"EG", N_("Egypt") },
 		{"EH", N_("Western Sahara") },
@@ -771,6 +857,7 @@ country (char *hostname)
 		{"GL", N_("Greenland") },
 		{"GM", N_("Gambia") },
 		{"GN", N_("Guinea") },
+		{"GOV", N_("Government") },
 		{"GP", N_("Guadeloupe") },
 		{"GQ", N_("Equatorial Guinea") },
 		{"GR", N_("Greece") },
@@ -789,6 +876,8 @@ country (char *hostname)
 		{"IE", N_("Ireland") },
 		{"IL", N_("Israel") },
 		{"IN", N_("India") },
+		{"INFO", N_("Informational") },
+		{"INT", N_("International") },
 		{"IO", N_("British Indian Ocean Territory") },
 		{"IQ", N_("Iraq") },
 		{"IR", N_("Iran") },
@@ -822,8 +911,10 @@ country (char *hostname)
 		{"MA", N_("Morocco") },
 		{"MC", N_("Monaco") },
 		{"MD", N_("Moldova") },
+		{"MED", N_("United States Medical") },
 		{"MG", N_("Madagascar") },
 		{"MH", N_("Marshall Islands") },
+		{"MIL", N_("Military") },
 		{"MK", N_("Macedonia") },
 		{"ML", N_("Mali") },
 		{"MM", N_("Myanmar") },
@@ -843,6 +934,7 @@ country (char *hostname)
 		{"NA", N_("Namibia") },
 		{"NC", N_("New Caledonia") },
 		{"NE", N_("Niger") },
+		{"NET", N_("Internic Network") },
 		{"NF", N_("Norfolk Island") },
 		{"NG", N_("Nigeria") },
 		{"NI", N_("Nicaragua") },
@@ -854,6 +946,7 @@ country (char *hostname)
 		{"NU", N_("Niue") },
 		{"NZ", N_("New Zealand") },
 		{"OM", N_("Oman") },
+		{"ORG", N_("Internic Non-Profit Organization") },
 		{"PA", N_("Panama") },
 		{"PE", N_("Peru") },
 		{"PF", N_("French Polynesia") },
@@ -870,10 +963,11 @@ country (char *hostname)
 		{"QA", N_("Qatar") },
 		{"RE", N_("Reunion") },
 		{"RO", N_("Romania") },
+		{"RPA", N_("Old School ARPAnet") },
 		{"RU", N_("Russian Federation") },
 		{"RW", N_("Rwanda") },
 		{"SA", N_("Saudi Arabia") },
-		{"Sb", N_("Solomon Islands") },
+		{"SB", N_("Solomon Islands") },
 		{"SC", N_("Seychelles") },
 		{"SD", N_("Sudan") },
 		{"SE", N_("Sweden") },
@@ -931,31 +1025,24 @@ country (char *hostname)
 		{"ZM", N_("Zambia") },
 		{"ZR", N_("Zaire") },
 		{"ZW", N_("Zimbabwe") },
-		{"COM", N_("Internic Commercial") },
-		{"EDU", N_("Educational Institution") },
-		{"GOV", N_("Government") },
-		{"INT", N_("International") },
-		{"MIL", N_("Military") },
-		{"NET", N_("Internic Network") },
-		{"ORG", N_("Internic Non-Profit Organization") },
-		{"RPA", N_("Old School ARPAnet") },
-		{"ATO", N_("Nato Fiel") },
-		{"MED", N_("United States Medical") },
-		{"ARPA", N_("Reverse DNS") },
-		{NULL, NULL}
 	};
 	char *p;
-	int i;
+	Domain *dom;
+
 	if (!hostname || !*hostname || isdigit (hostname[strlen (hostname) - 1]))
 		return _("Unknown");
 	if ((p = strrchr (hostname, '.')))
 		p++;
 	else
 		p = hostname;
-	for (i = 0; domain[i].code; i++)
-		if (!strcasecmp (p, domain[i].code))
-			return domain[i].country;
-	return _("Unknown");
+
+	dom = bsearch (p, domain, sizeof (domain) / sizeof (Domain),
+						sizeof (Domain), country_compare);
+
+	if (!dom)
+		return _("Unknown");
+
+	return _(dom->country);
 }
 
 /* I think gnome1.0.x isn't necessarily linked against popt, ah well! */
@@ -1110,13 +1197,13 @@ make_ping_time (void)
  */
 
 int
-rfc_casecmp (char *s1, char *s2)
+rfc_casecmp (const char *s1, const char *s2)
 {
 	register unsigned char *str1 = (unsigned char *) s1;
 	register unsigned char *str2 = (unsigned char *) s2;
 	register int res;
 
-	while ((res = tolower (*str1) - tolower (*str2)) == 0)
+	while ((res = rfc_tolower (*str1) - rfc_tolower (*str2)) == 0)
 	{
 		if (*str1 == '\0')
 			return 0;
@@ -1133,7 +1220,7 @@ rfc_ncasecmp (char *str1, char *str2, int n)
 	register unsigned char *s2 = (unsigned char *) str2;
 	register int res;
 
-	while ((res = tolower (*s1) - tolower (*s2)) == 0)
+	while ((res = rfc_tolower (*s1) - rfc_tolower (*s2)) == 0)
 	{
 		s1++;
 		s2++;
@@ -1144,7 +1231,7 @@ rfc_ncasecmp (char *str1, char *str2, int n)
 	return (res);
 }
 
-unsigned char tolowertab[] =
+const unsigned char rfc_tolowertab[] =
 	{ 0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
 	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
 	0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
@@ -1213,3 +1300,169 @@ unsigned char tolowertab[] =
 	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };*/
+
+/* Code borrowed from gtk-gnutella */
+/* Takes care of moving a file from a temporary download location to a completed location */
+void
+download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir, 
+	char *output_name, int dccpermissions)
+{
+	/* Move a complete file to move_file_path */
+	/* mgl: this used to take just a filename and move it between dirs */
+	/* now it takes the full path of the target download and moves it */
+
+	char dl_src[4096];
+	char dl_dest[4096];
+	char dl_tmp[4096];
+	char *output_fname;
+	int return_tmp, return_tmp2;
+
+	/* if dcc_dir and dcc_completed_dir are the same then we are done */
+	if (0 == strcmp (dcc_dir, dcc_completed_dir) ||
+		 0 == dcc_completed_dir[0])
+		return;			/* Already in "completed dir" */
+
+	/* mgl: since output_name is a full path, we just copy it */
+	strncpy (dl_src, output_name, sizeof(dl_src));
+	dl_src[sizeof(dl_src)-1] = '\0';
+
+	/* mgl: output_name being a full path, we need to extract the filename */
+	/* off the end of it before continuing */
+
+	/* no path sep or no file after pathsep?  very suspicious, bail now! */
+	if ((NULL == (output_fname = strrchr(output_name, '/')))
+			|| !*(output_fname + 1))
+		return;
+	/* get the next char after the pathsep */
+	++output_fname;
+	/* throw the filename onto the directory name of the completed directory */
+	/* FIXME: dcc_completed_dir is UTF8, not fs encoding! */
+	snprintf (dl_dest, sizeof (dl_dest), "%s/%s", dcc_completed_dir,
+			   output_fname);
+	/* the rest should continue as before, but use output_fname */
+
+	dl_dest[sizeof(dl_dest)-1] = '\0';
+
+	/*
+	 * If, by extraordinary, there is already a file in the "completed dir"
+	 * with the same name, don't overwrite the existing file.
+	 *
+	 * NB: we assume either there is only one gnutella servent running, or if
+	 * several ones are running, that they are configured to use different
+	 * download and completed dirs.
+	 *
+	 *		--RAM, 03/11/2001
+	 */
+
+	if (access (dl_dest, F_OK) == 0)
+	{
+		int destlen = strlen (dl_dest);
+		int i;
+		struct stat buf;
+
+		/*
+		 * There must be enough room for us to append the ".xx" extensions.
+		 * That's 3 chars, plus the trailing NUL.
+		 */
+
+		if (destlen >= sizeof (dl_dest) - 4)
+		{
+			fprintf(stderr, "Found '%s' in completed dir, and path already too long",
+				output_fname);
+			return;
+		}
+
+		strncpy (dl_tmp, dl_dest, destlen);
+
+		for (i = 1; i < 100; i++)
+		{
+			char ext[4];
+
+			snprintf (ext, 4, ".%02d", i);
+			dl_tmp[destlen] = '\0';				/* Ignore prior attempt */
+			strncat (dl_tmp+destlen, ext, 3);	/* Append .01, .02, ...*/
+			if (-1 == stat (dl_tmp, &buf))
+				break;
+		}
+
+		if (i == 100)
+		{
+			fprintf (stderr, "Found '%s' in completed dir, "
+				"and was unable to find another unique name",
+				output_fname);
+			return;
+		}
+
+		strncat (dl_dest+destlen, dl_tmp+destlen, 3);
+	}
+
+	/* First try and link it to the new locatation */
+
+	return_tmp = rename (dl_src, dl_dest);
+
+	if (return_tmp == -1 && (errno == EXDEV || errno == EPERM))
+	{
+		/* link failed becase either the two paths aren't on the */
+		/* same filesystem or the filesystem doesn't support hard */
+		/* links, so we have to do a copy. */
+
+		int tmp_src, tmp_dest;
+		gboolean ok = FALSE;
+
+		if ((tmp_src = open (dl_src, O_RDONLY | OFLAGS)) == -1)
+		{
+			fprintf (stderr, "Unable to open() file '%s' (%s) !", dl_src,
+					  strerror (errno));
+			return;
+		}
+
+		if ((tmp_dest =
+			 open (dl_dest, O_WRONLY | O_CREAT | O_TRUNC | OFLAGS, dccpermissions)) < 0)
+		{
+			close (tmp_src);
+			fprintf (stderr, "Unable to create file '%s' (%s) !", dl_src,
+					  strerror (errno));
+			return;
+		}
+
+		for (;;)
+		{
+			return_tmp = read (tmp_src, dl_tmp, sizeof (dl_tmp));
+
+			if (!return_tmp)
+			{
+				ok = TRUE;
+				break;
+			}
+
+			if (return_tmp < 0)
+			{
+				fprintf (stderr, "download_move_to_completed_dir(): "
+					"error reading while moving file to save directory (%s)",
+					 strerror (errno));
+				break;
+			}
+
+			return_tmp2 = write (tmp_dest, dl_tmp, return_tmp);
+
+			if (return_tmp2 < 0)
+			{
+				fprintf (stderr, "download_move_to_completed_dir(): "
+					"error writing while moving file to save directory (%s)",
+					 strerror (errno));
+				break;
+			}
+
+			if (return_tmp < sizeof(dl_tmp))
+			{
+				ok = TRUE;
+				break;
+			}
+		}
+
+		close (tmp_dest);
+		close (tmp_src);
+		if (ok)
+			unlink (dl_src);
+	}
+}
